==> linstor-server-1.25.0.tar.gz: linstor-server-1.25.0/controller/src/main/java/com/linbit/linstor/dbcp/migration/k8s/crd/Migration_16_v1_19_1_RestoreInvisbleKvs.java <==
package com.linbit.linstor.dbcp.migration.k8s.crd;

import com.linbit.linstor.ControllerK8sCrdDatabase;
import com.linbit.linstor.dbdrivers.GeneratedDatabaseTables;
import com.linbit.linstor.dbdrivers.k8s.crd.GenCrdV1_19_1;
import com.linbit.linstor.dbdrivers.k8s.crd.GenCrdV1_19_1.KeyValueStore;
import com.linbit.linstor.dbdrivers.k8s.crd.GenCrdV1_19_1.KeyValueStoreSpec;
import com.linbit.linstor.dbdrivers.k8s.crd.GenCrdV1_19_1.PropsContainers;
import com.linbit.linstor.dbdrivers.k8s.crd.GenCrdV1_19_1.PropsContainersSpec;

import java.util.Collection;
import java.util.HashSet;
import java.util.UUID;

@K8sCrdMigration(
    description = "Cleanup orphaned ACL entries",
    version = 16
)
public class Migration_16_v1_19_1_RestoreInvisbleKvs extends BaseK8sCrdMigration
{
    /*
     * Migration_10_v1_19_1_CleanupOrphanedObjects accidentally deleted all KVS entries from the KVS table, but not
     * the props.
     * The problem was that the migration scanned for KVS with no properties, but was looking for lower-cased
     * PROPS_INSTANCE entries although the driver uppercases the PROPS_INSTANCE always.
     *
     * The props were not touched, but since the entries in the KVS table was missing,
     * "linstor key-value-store list" did not list them and "linstor key-value-store show <kvs>" also did not
     * contain any entries since they were simply not loaded from the database
     */

    private static final String PROP_INSTANCE_PREFIX = "/KEYVALUESTORES/";

    public Migration_16_v1_19_1_RestoreInvisbleKvs()
    {
        super(
            GenCrdV1_19_1.createTxMgrContext(),
            GenCrdV1_19_1.createTxMgrContext(),
            GenCrdV1_19_1.createSchemaUpdateContext()
        );
    }

    @Override
    public MigrationResult migrateImpl(ControllerK8sCrdDatabase k8sDbRef) throws Exception
    {
        HashSet<String> kvsFromProps = new HashSet<>();

        Collection<PropsContainers> propsCrdList = txFrom.<PropsContainers, PropsContainersSpec>getCrd(
            GeneratedDatabaseTables.PROPS_CONTAINERS
        ).values();

        for (PropsContainers props : propsCrdList)
        {
            PropsContainersSpec spec = props.getSpec();
            if (spec.propsInstance.startsWith(PROP_INSTANCE_PREFIX))
            {
                kvsFromProps.add(spec.propsInstance.substring(PROP_INSTANCE_PREFIX.length()));
            }
        }

        Collection<KeyValueStore> kvsCrdList = txFrom.<KeyValueStore, KeyValueStoreSpec>getCrd(
            GeneratedDatabaseTables.KEY_VALUE_STORE
        ).values();
        for (KeyValueStore kvs : kvsCrdList)
        {
            kvsFromProps.remove(kvs.getSpec().kvsName);
        }

        for (String kvsToRestore : kvsFromProps)
        {
            txTo.create(
                GeneratedDatabaseTables.KEY_VALUE_STORE,
                GenCrdV1_19_1.createKeyValueStore(
                    UUID.randomUUID().toString(),
                    kvsToRestore,
                    kvsToRestore.toLowerCase()
                )
            );
        }

        return null;
    }
}

==> linstor-server-1.25.1.tar.gz: linstor-server-1.25.1/controller/src/main/java/com/linbit/linstor/dbcp/migration/k8s/crd/Migration_16_v1_19_1_RestoreInvisbleKvs.java <==
package com.linbit.linstor.dbcp.migration.k8s.crd;

import com.linbit.linstor.ControllerK8sCrdDatabase;
import com.linbit.linstor.dbdrivers.GeneratedDatabaseTables;
import com.linbit.linstor.dbdrivers.k8s.crd.GenCrdV1_19_1;
import com.linbit.linstor.dbdrivers.k8s.crd.GenCrdV1_19_1.KeyValueStore;
import com.linbit.linstor.dbdrivers.k8s.crd.GenCrdV1_19_1.KeyValueStoreSpec;
import com.linbit.linstor.dbdrivers.k8s.crd.GenCrdV1_19_1.PropsContainers;
import com.linbit.linstor.dbdrivers.k8s.crd.GenCrdV1_19_1.PropsContainersSpec;

import java.util.Collection;
import java.util.HashSet;
import java.util.UUID;

@K8sCrdMigration(
    description = "Restore invisible KVS",
    version = 16
)
public class Migration_16_v1_19_1_RestoreInvisbleKvs extends BaseK8sCrdMigration
{
    /*
     * Migration_10_v1_19_1_CleanupOrphanedObjects accidentally deleted all KVS entries from the KVS table, but not
     * the props.
     * The problem was that the migration scanned for KVS with no properties, but was looking for lower-cased
     * PROPS_INSTANCE entries although the driver uppercases the PROPS_INSTANCE always.
     *
     * The props were not touched, but since the entries in the KVS table was missing,
     * "linstor key-value-store list" did not list them and "linstor key-value-store show <kvs>" also did not
     * contain any entries since they were simply not loaded from the database
     */

    private static final String PROP_INSTANCE_PREFIX = "/KEYVALUESTORES/";

    public Migration_16_v1_19_1_RestoreInvisbleKvs()
    {
        super(
            GenCrdV1_19_1.createTxMgrContext(),
            GenCrdV1_19_1.createTxMgrContext(),
            GenCrdV1_19_1.createSchemaUpdateContext()
        );
    }

    @Override
    public MigrationResult migrateImpl(ControllerK8sCrdDatabase k8sDbRef) throws Exception
    {
        HashSet<String> kvsFromProps = new HashSet<>();

        Collection<PropsContainers> propsCrdList = txFrom.<PropsContainers, PropsContainersSpec>getCrd(
            GeneratedDatabaseTables.PROPS_CONTAINERS
        ).values();

        for (PropsContainers props : propsCrdList)
        {
            PropsContainersSpec spec = props.getSpec();
            if (spec.propsInstance.startsWith(PROP_INSTANCE_PREFIX))
            {
                kvsFromProps.add(spec.propsInstance.substring(PROP_INSTANCE_PREFIX.length()));
            }
        }

        Collection<KeyValueStore> kvsCrdList = txFrom.<KeyValueStore, KeyValueStoreSpec>getCrd(
            GeneratedDatabaseTables.KEY_VALUE_STORE
        ).values();
        for (KeyValueStore kvs : kvsCrdList)
        {
            kvsFromProps.remove(kvs.getSpec().kvsName);
        }

        for (String kvsToRestore : kvsFromProps)
        {
            txTo.create(
                GeneratedDatabaseTables.KEY_VALUE_STORE,
                GenCrdV1_19_1.createKeyValueStore(
                    UUID.randomUUID().toString(),
                    kvsToRestore,
                    kvsToRestore.toLowerCase()
                )
            );
        }

        return null;
    }
}
